@@ -1,6 +1,5 @@
 if(require("testthat", quietly = TRUE)) {
-    #test_check("nloptr")
-    #test_package("nloptr")
+    test_check("nloptr")
 } else {
     print( "package 'testthat' not available, cannot run unit tests" )
 }
